"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
const semver = require("semver");
const util_constants_1 = require("util.constants");
const json = require("jju");
const intersect = require("semver-set").intersect;
const handlers = {
    keywords: unique,
    scripts: exists,
    dependencies: updateDependencies,
    devDependencies: updateDependencies,
    peerDependencies: updateDependencies
};
function exists(dst, src) {
    return Object.assign({}, dst, src);
}
function unique(dst = [], src = []) {
    const set = new Set(dst);
    src.forEach((it) => {
        set.add(it);
    });
    return Array.from(set);
}
function updateDependencies(dst, src) {
    return _.isEmpty(dst)
        ? src
        : Object.assign({}, dst, _.mapValues(src, (version, dep) => {
            // We need to check if both are indeed semver ranges in order to do
            // intersects – some may be git urls or other such things.
            const isSem = semver.validRange(version) &&
                semver.validRange(dst[dep]);
            return isSem
                ? intersect(version, dst[dep]) || version
                : version;
        }));
}
function combine(dst, src) {
    return _.isEmpty(dst)
        ? src
        : Object.assign({}, dst, _.mapValues(src, (value, key) => {
            return _.has(handlers, key)
                ? handlers[key](dst[key], value)
                : value;
        }));
}
/**
 * Takes two input objects and attempts to merge them together.  This function
 * should be used for package.json files.  This function will take special care
 * to merge dependencies (and their versions), scripts, and keyword sections
 * of the package.json file.
 * @param dst {string|Buffer|object} the target object for the merge
 * @param src {string|Buffer|object} the source object for the merge
 * @returns {string} a merged object string in JSON format.
 */
function merge(dst, src) {
    let odst = dst;
    if (dst instanceof Buffer) {
        dst = dst.toString(util_constants_1.encoding);
    }
    if (src instanceof Buffer) {
        src = src.toString(util_constants_1.encoding);
    }
    if (typeof dst === "object") {
        odst = JSON.stringify(dst);
    }
    if (typeof dst === "string") {
        dst = JSON.parse(dst);
    }
    if (typeof src === "string") {
        src = JSON.parse(src);
    }
    if (_.isEqual(dst, src)) {
        return odst;
    }
    return json.update(odst, combine(dst, src), {});
}
exports.default = merge;
//# sourceMappingURL=index.js.map